 /*
  * sigact.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>   /* dirname  */
#include <signal.h>
#include <unistd.h>
#include <ftw.h>          /*  nftw  */
#include <sys/un.h>      /*  UNIX socket  */
#include <sys/types.h>  
#include <sys/socket.h>
#include <assert.h>
#include <inttypes.h>
#include <ctype.h>       /* ???   */

#include <simple-netaid/sbuf.h>
#include <simple-netaid/helpers.h>
#include <simple-netaid/interfaces.h>
#include <simple-netaid/ipaddr.h>
#include <simple-netaid/wireless.h>
#include <simple-netaid/iproute.h>

#include "sigact.h"
#include "start_periodic.h"
#include "def.h"
#include "config.h"
#include "proc.h"

#define MAX_SIZE 1024

static
void sig_handler(int signum, siginfo_t *info, void *extra)
{ 
   switch(signum) {
      
      case SIGALRM:
         break;
        
      case SIGHUP:
         break;
        
      case SIGUSR1:
         break;
         
      case SIGTERM:
      case SIGINT:
      case SIGQUIT:
         { 
           int count __attribute__((unused)) = 0;       
       
           // rt_netlink monitoring part
           if( access( m_config->nl_monitor_path, F_OK ) == 0 ) {
	          char *dir_name = dirname(strdup(m_config->nl_monitor_path));
	          char *base_name = basename(strdup(m_config->nl_monitor_path));
	          char *aux = NULL;
	          char path[64] = {0};
	          	                   
              unlink(m_config->nl_monitor_path);	
	          
	          // Remove the monitor directory only if it's a subdir of tmp
	          // We do not want to rely on the use of 'tmpreaper', similar 
	          // to 'tmpwatch' in redhat and derivatives, aimed to cleanup 
	          // empty directories in /tmp
	          aux = rindex(dir_name, '/');
	          if(strcmp(aux, "/tmp")) {
	             sprintf(path, "/tmp/");
	             strcat(path, aux+1);
	             if(!strcmp(path, dir_name)) {
	                rmdir(dir_name);
	             }
	          }
    
              free(dir_name);
              // NOTE: 'base_name' and 'aux' are freed
           } 
        
           count = termproc(60);
                
           // Ubus loop part
           if( access( m_config->pidfile_path, F_OK ) == 0 ) {
              uloop_done();
              ubus_free(ctx);
              if(pid_fd != -1) {
                 int rc = lockf(pid_fd, F_ULOCK, 0);
                 if(rc != 0) exit(EXIT_FAILURE);
                 close(pid_fd);
              }        
              unlink(m_config->pidfile_path);
           } 
         }
         
         exit(EXIT_SUCCESS);  /* exit gracefully. */
         break;
         
      default:
         break;
   }
}

void sigaction_init(void)
{
   struct sigaction sa;
   memset (&sa, 0, sizeof(sa));
     
   /*---------------- initialize signal handling ------------ */
   //sa.sa_handler = sig_handler;   
   sa.sa_sigaction = &sig_handler;
   sa.sa_flags = SA_RESETHAND | SA_NODEFER | SA_SIGINFO | SA_RESTART;
   sigemptyset(&sa.sa_mask);
   sigaction(SIGALRM, &sa, 0);
   sigaction(SIGHUP,  &sa, 0);
   sigaction(SIGUSR1, &sa, 0);
   sigaction(SIGTERM, &sa, 0);
   sigaction(SIGINT,  &sa, 0);
   sigaction(SIGQUIT, &sa, 0);
}
